# Generated by Django 4.0.8 on 2023-12-13 16:41

from django.db import migrations, models


def echo(*args, **kwargs):
    print('This migration will be deleted in the next release. If you already applied previous version, drop supplier_is_vat_payer field manually!')

class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0029_auto_20230912_0458'),
    ]

    operations = [
        # TODO: DEPRECATED: previous version of migration adding new field
        # migrations.AddField(
        #     model_name='invoice',
        #     name='supplier_is_vat_payer',
        #     field=models.BooleanField(blank=True, null=True, default=None)
        # ),
        migrations.RunPython(echo, migrations.RunPython.noop)
    ]
