# Generated by Django 2.2.9 on 2020-09-01 10:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0023_invoice_related_invoices'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='date_reminder_sent',
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
    ]
