# Generated by Django 2.2.23 on 2023-09-12 02:04

from django.db import migrations, models
import internationalflavor.vat_number.models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0027_auto_20230207_2029'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='already_paid',
            field=models.DecimalField(decimal_places=2, default=0, max_digits=10),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='customer_vat_id',
            field=internationalflavor.vat_number.models.VATNumberField(blank=True),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='related_invoices',
            field=models.ManyToManyField(blank=True, related_name='_invoice_related_invoices_+', to='invoicing.Invoice'),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='supplier_vat_id',
            field=internationalflavor.vat_number.models.VATNumberField(blank=True),
        ),
    ]
