# Generated by Django 2.0.3 on 2018-05-29 10:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0005_auto_20171026_0009'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='invoice',
            options={
                'default_permissions': ('list', 'view', 'add', 'change', 'delete'),
                'ordering': ('date_issue', 'number'),
                'verbose_name': 'invoice',
                'verbose_name_plural': 'invoices'
            },
        ),
    ]
