# Generated by Django 2.1.10 on 2019-09-12 06:53

from django.db import migrations
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0020_auto_20181001_1025'),
        ('invoicing', '0021_invoice_related_document'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='date_paid',
            field=model_utils.fields.MonitorField(blank=True, default=None, monitor='status', null=True, when={'PAID'}),
        ),
    ]
