# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2017-05-22 10:07
from __future__ import unicode_literals

from django.db import migrations, models
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0003_auto_20170123_1810'),
    ]

    operations = [
        migrations.AlterField(
            model_name='invoice',
            name='date_sent',
            field=model_utils.fields.MonitorField(blank=True, default=None, monitor='status', null=True, when=set(['SENT'])),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='delivery_method',
            field=models.CharField(default='PERSONAL_PICKUP', max_length=64),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='status',
            field=models.CharField(default='NEW', max_length=64),
        ),
        migrations.AlterField(
            model_name='invoice',
            name='type',
            field=models.CharField(default='INVOICE', max_length=64),
        ),
        migrations.AlterField(
            model_name='item',
            name='unit',
            field=models.CharField(default='PIECES', max_length=64),
        ),
    ]
