# Generated by Django 2.0.3 on 2018-06-01 12:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoicing', '0010_recalculate_vat'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='title',
            field=models.TextField(),
        ),
    ]
